/* check for unknown NIC ids and drivers */

Trace 'N'

/* get our directory */
Parse Source . . our_cmd .
our_dir = Left(our_cmd, Lastpos('\', our_cmd) - 1)
Call Directory(our_dir)
nic_lst = our_dir || '\nicdrv.lst'
nic_txt = our_dir || '\nictst.txt'
res_view = '@rmview /dc'
res_view = '@pci'

/* Load RexxUtil.DLL */
Call Rxfuncadd 'SysLoadFuncs','Rexxutil','SysLoadFuncs'
Call SysLoadFuncs

Parse Arg pmrexx
pmrexx = (pmrexx <> '')
If \pmrexx Then
   Do
      /* setup to run under PMREXX */
      pmrexx_name = SysSearchPath('PATH', 'PMREXX.EXE')
      If pmrexx_name <> '' Then
         Do
            class = 'WPProgram'
            title = 'NIC Test'
            location = '<WP_DESKTOP>'
            obj = 'OBJECTID=<NIC_TST>;'
            exec = 'EXENAME=' || pmrexx_name || ';'
            parm = 'PARAMETERS=' || our_cmd 'X' || ';'
            startdir = 'STARTUPDIR=' || our_dir || ';'
            setup = obj || exec || parm || startdir
            rc = SysCreateObject(class, title, location, setup, 'R')
            If rc Then
               Do
                  Say 'Starting NIC Test under PMRexx'
                  rc = SysSetObjectData('<NIC_TST>', 'OPEN=DEFAULT;CCVIEW=NO;')
                  If rc Then
                     Say 'Started NIC Test under PMRexx'
                  Return 0
               End
            Else
               Say 'Unable to start NIC Test under PMRexx'
         End
   End

Call SysFileDelete(nic_txt)

Call Query_devices                            /* build device id list */

Call Check_devices                           /* check for NIC devices */

/* locate \IBMCOM\MACS */
ibm_macs = SysSearchPath('path', 'protocol.ini')
ibm_macs = Left(ibm_macs, Lastpos('\', ibm_macs) -1)
ibm_macs = ibm_macs || '\MACS'

/* select a NIC */
If nic_not_found.0 <> 0 Then
   Do
      Call Load_nic_table 'unknown pcmcia isa'
      Do i = 1 To nic_list.0
         Call Nic_load nic_list.i
      End
      Do i = 1 To nic_not_found.0
         Call Lineout nic_txt, nic_not_found.i
         Say 'I was not able to identify the following NIC on your PCI bus'
         Say 'Vendor =' Left(nic_not_found.i, 4) 'Device =' Substr(nic_not_found.i, 5)
         nic_select = Select_nic()             /* select a NIC driver */
         If Datatype(nic_select, 'w') & nic_no.0 >= nic_select & nic_select >= 1 Then
            Do
               Call Lineout nic_txt, nic_title.nic_select
            End
         Else
            Do
               Say 'Please enter a one line description of your card'
               Parse Pull card_desc
               If card_desc = '' Then
                  card_desc = 'No reply'
               Call Lineout nic_txt, card_desc
            End
      End
      Say 'The results of this test have been written in' nic_txt
      Say 'Please attach' nic_txt 'to an e_mail and send to'
      Say 'Chuck McKinnis <mckinnis@attglobal.net>'
      Say 'Thanks for helping make eCS a better solution.'
   End
Else
   Do
      If bus_device_id.0 <> 0 Then
         Do
            Say 'I was able to identify all of your NIC(s) on your PCI bus'
            Say 'Thanks for helping make eCS a better solution.'
         End
      Else
         Say 'No network cards were detected in any PCI slot'
   End
Say 'Press enter to exit'
Parse Pull .
If pmrexx Then
   Do
      Call SysSleep 2
      Call SysDestroyObject '<NIC_TST>'
   End
Return

/* query devices */
Query_devices:
save_trace = Trace('N')
dev_queue = Rxqueue('create')
old_queue = Rxqueue('set', dev_queue)
Address cmd res_view '| rxqueue' dev_queue

/* get device ids */
bus_device_id. = ''
bus_device_id.0 = 0
Do i = 1 Until Queued() = 0
   nic_no = ''
   Parse Pull line
   line = Space(line)
   Select
      When Abbrev(res_view, '@pci') Then    /* handle output from PCI */
         Do
            Parse Var line num '|' bus_vendor_id '|' bus_device_id '|' bus_class '|' .
            If Datatype(num, 'W') & bus_class = 200 Then
               Do
                  bus_device_id = Space(bus_vendor_id) || Space(bus_device_id)
                  j = bus_device_id.0 + 1
                  bus_device_id.j = Space(bus_device_id)
                  bus_device_id.0 = j
               End
            Iterate i
         End
      When Abbrev(res_view, '@rmv') Then /* handle output from RMVIEW */
         Do
            Select
               When Abbrev(line, 'PCI Device ID:') Then
                  Do
                     Parse Var line . ':' bus_device_id
                     j = bus_device_id.0 + 1
                     bus_device_id.j = Space(bus_device_id)
                     bus_device_id.0 = j
                  End
               When Abbrev(line, 'PnP Device ID:') Then
                  Do
                     Parse Var line . ':' bus_device_id
                     j = bus_device_id.0 + 1
                     bus_device_id.j = Space(bus_device_id)
                     bus_device_id.0 = j
                  End
               Otherwise Iterate i
            End
         End
      Otherwise Nop
   End
End
Call Rxqueue 'set', old_queue
Call Rxqueue 'delete', new_queue
Trace(save_trace)
Return

/* check the ids found */
Check_devices:
save_trace = Trace('N')

Call Clear_nic_table                               /* clear NIC table */

/* scan for NIC cards */
nic_not_found. = ''
nic_not_found.0 = 0
Do i = 1 To bus_device_id.0
   test = bus_device_id.i
   rc = SysFileSearch(test, nic_lst, 'nic_found.')
   If rc = 0 & nic_found.0 = 0 Then
      Do
         j = nic_not_found.0 + 1
         nic_not_found.j = bus_device_id.i
         nic_not_found.0 = j
      End
End
Trace(save_trace)
Return

/* clear NIC table */
Clear_nic_table:
save_trace = Trace('N')
nic_no. = ''
nic_no.0 = 0
nic_title. = ''
nic_title.0 = 0
max_title = 0
nic_dir. = ''
nic_dir.0 = 0
max_dir = 0
nic_nif. = ''
nic_nif.0 = 0
max_nif = 0
Trace(save_trace)
Return

/* get NIC table info */
Nic_load:
save_trace = Trace('N')
Parse Arg nic_found
Parse Var nic_found nic_no '|' nic_nif '|' nic_title
nic_dir = Left(nic_nif, Lastpos('\', nic_nif) - 1)
nic_nif = Substr(nic_nif, Lastpos('\', nic_nif) + 1)
k = nic_no.0 + 1
nic_no.k = nic_no
nic_no.0 = k
k = nic_title.0 + 1
nic_title.k = Space(nic_title)
nic_title.0 = k
If Length(nic_title.k) > max_title Then
   max_title = Length(nic_title.k)
k = nic_dir.0 + 1
nic_dir.k = Space(nic_dir)
nic_dir.0 = k
If Length(nic_dir.k) > max_dir Then
   max_dir = Length(nic_nif.k)
k = nic_nif.0 + 1
nic_nif.k = Space(nic_nif)
nic_nif.0 = k
If Length(nic_nif.k) > max_nif Then
   max_nif = Length(nic_nif.k)
Trace(save_trace)
Return

/* select a NIC driver */
Select_nic:
save_trace = Trace('n')
Parse Arg detail .
detail = Abbrev(detail, 'D')
max_no = Length(nic_no.0)
screen_list. = ''
screen_list.0 = 0
x = 0
Do i = 1 To nic_no.0
   If detail | i = 1 Then
      Do
         x = x + 1; screen_list.x = Copies(' ', max_no) '  Card Number  ' 'Chip Mfg - Card Title'
      End
   x = x + 1; screen_list.x = Right(i, max_no) '-' Left(nic_no.i, Length('Card Number') + 2) Left(nic_title.i, max_title)
   If Length('NIF File') > max_nif Then
      max_nif = Length('NIF File')
   If detail Then
      Do
         x = x + 1; screen_list.x = Copies(' ', max_no + 2) Left('NIF File', max_nif + 1) 'NIF & Driver Information'
         x = x + 1; screen_list.x = Copies(' ', max_no + 2) Left(nic_nif.i, max_nif + 1) ibm_macs_nif
         x = x + 1; screen_list.x = Copies(' ', max_no + 2) Copies(' ', max_nif + 1) ibm_macs_driver
         x = x + 1; screen_list.x = Copies(' ', max_no + 2) Copies(' ', max_nif + 1) nic_macs_nif
         x = x + 1; screen_list.x = Copies(' ', max_no + 2) Copies(' ', max_nif + 1) nic_macs_driver
         x = x + 1; screen_list.x = ''
      End
End
screen_list.0 = x
Do i = 1 To screen_list.0
   Say screen_list.i
End
Say 'Enter the number of the NIC that describes your card, or press enter to bypass'
Parse Upper Pull ans .
Trace(save_trace)
Return ans

/* load the NIC device table */
Load_nic_table:
save_trace = Trace('N')
Parse Arg entry_types
Call Clear_nic_table
nic_list. = ''
nic_list.0 = 0
Do i = 1 While Lines(nic_lst)
   nic_info = Linein(nic_lst)
   If Wordpos(Word(nic_info, 1), entry_types) <> 0 Then
      Do
         j = nic_list.0 + 1
         nic_list.j = nic_info
         nic_list.0 = j
      End
End
Trace(save_trace)
Return
